﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web.UI;

namespace ELTE.Web.GoogleStaticMap.Control
{
    /// <summary>
    /// Térkép megjelenésének felsorolási típusa.
    /// </summary>
    public enum MapType { RoadMap, Satellite, Hybrid, Terrain }

    /// <summary>
    /// Google statikus térkép típusa.
    /// </summary>
    [Serializable]
    [ToolboxItem(true)]
    [ToolboxData("<{0}:GoogleStaticMap runat=\"server\"></{0}:GoogleStaticMap>")]
    public class GoogleStaticMap : System.Web.UI.WebControls.Image // öse egy képmegjelenítő webes vezérlő
    {
        private GoogleCoordinate _Center;
        private Boolean _AutoCenter;
        private Boolean _AutoZoom;
        private List<GoogleMarker> _Markers;
        private List<GooglePath> _Paths;

        /// <summary>
        /// Nagyítás mértékének lekérdezése, vagy beállítása.
        /// </summary>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("10")]
        [Localizable(false)]
        public Int32 ZoomLevel { get; set; }

        /// <summary>
        /// Térkép megjelnésének lekérdezése, vagy beállítása.
        /// </summary>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("RoadMap")]
        [Localizable(false)]
        public MapType MapType { get; set; }

        /// <summary>
        /// Térkép kulcsának lekérdezése, vagy beállítása.
        /// </summary>
        [Bindable(true)]
        [Category("Google")]
        [DefaultValue("")]
        [Localizable(false)]
        public String Key { get; set; }

        /// <summary>
        /// Automatikus középigazítás meglétének lekérdezése, vagy beállítása.
        /// </summary>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Localizable(false)]
        public Boolean AutoCenter
        {
            get { return _AutoCenter && IsAutoAvailable; } // csak akkor működik, ha legalább egy jelölő van a térképen
            set { _AutoCenter = value; }
        }

        /// <summary>
        /// Automatikus nagyítás meglétének lekérdezése, vagy beállítása.
        /// </summary>
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Localizable(false)]
        public bool AutoZoom
        {
            get { return _AutoZoom && IsAutoAvailable; }
            set { _AutoZoom = value; }
        }

        /// <summary>
        /// Automatikus tájolási lehetőségek meglétének lekérdezése.
        /// </summary>
        public Boolean IsAutoAvailable
        {
            get { return (Markers.Count > 0 || Paths.Count > 0); }
        }

        /// <summary>
        /// Térkép középpontjának lekérdezése.
        /// </summary>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)] // a kódgenerálási mód megadása
        [PersistenceMode(PersistenceMode.InnerProperty)] // tervezési nézetben való megjelenés 
        public GoogleCoordinate Center
        {
            get
            {
                if (_Center == null)
                {
                    _Center = new GoogleCoordinate(0, 0);
                }
                return _Center;
            }
            set
            {
                _Center = value;
            }
        }

        /// <summary>
        /// Térképi jelölések lekérdezése.
        /// </summary>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        public List<GoogleMarker> Markers
        {
            get { return _Markers; }
        }

        /// <summary>
        /// Térképi útvonalak lekérdezése.
        /// </summary>
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        public List<GooglePath> Paths
        {
            get { return _Paths; }
        }

        public GoogleStaticMap() 
        {
            _Markers = new List<GoogleMarker>();
            _Paths = new List<GooglePath>();
        }

        protected override void LoadViewState(object savedState) // nézetállapot betöltése
        {
            base.LoadViewState(savedState);

            Key = (String)ViewState["Key"]; // a kulcsot  a nézetállapotból vonjuk ki
            ZoomLevel = (Int32)ViewState["ZoomLevel"];
            _Center = (GoogleCoordinate)ViewState["Center"];            
            _Markers.AddRange(new List<GoogleMarker>((GoogleMarker[])ViewState["Markers"]));
            _Paths.AddRange(new List<GooglePath>((GooglePath[])ViewState["Paths"]));
            _AutoCenter = (Boolean)ViewState["AutocCenter"];
            _AutoZoom = (Boolean)ViewState["AutoZoomLevel"];
        }

        protected override object SaveViewState() // nézetállapot elmentése
        {
            ViewState["Center"] = _Center;
            ViewState["ZoomLevel"] = ZoomLevel;
            ViewState["Markers"] = _Markers.ToArray();
            ViewState["Paths"] = _Paths.ToArray();
            ViewState["Key"] = Key;
            ViewState["AutomaticCenter"] = _AutoCenter;
            ViewState["AutomaticZoomLevel"] = _AutoZoom;

            return base.SaveViewState();
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            base.ImageUrl = BuildUrl(); // az elérési útvonalat a már definiált szöveggé alakítások segítségével építjük össze
        }

        private String BuildUrl()
        {
            String url = "http://maps.google.com/staticmap?"; // url felépítése, amely lekéri a megfelelő képet

            List<String> parameters = new List<String>();

            if (!_AutoCenter && _Center != null)
            {
                parameters.Add("center=" + _Center.ToString());
            }

            if (!_AutoZoom)
            {
                parameters.Add("zoom=" + ZoomLevel);
            }

            parameters.Add("maptype=" + Enum.GetName(typeof(MapType), MapType).ToLower());

            parameters.Add("size=" + Convert.ToInt32(Width.Value) + "x" + Convert.ToInt32(Height.Value));

            if (_Markers.Count > 0)
            {
                List<String> markers = new List<string>();
                foreach (GoogleMarker marker in _Markers)
                {
                    markers.Add(marker.ToString());
                }
                parameters.Add("markers=" + String.Join("|", markers.ToArray()));
            }

            if (_Paths.Count > 0)
            {
                foreach (GooglePath path in _Paths)
                {
                    parameters.Add(path.ToString());
                }
            }

            if (Key != String.Empty)
            {
                parameters.Add("key=" + Key.ToLower());
            }

            return url + String.Join("&", parameters.ToArray()) + "&sensor=false";
        }
    }
}
