﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ELearning.Models.Exceptions;
using System.Security.Cryptography;
using System.IO;
using System.Drawing;

namespace ELearning.Models
{
    /// <summary>
    /// Adatbázis absztrakciós réteg
    /// </summary>
    internal static class DataManager
    {
        private static ELearningEntities _Model;
        private const String _PasswordSalt = "ElearningRulez2012";

        /// <summary>
        /// Statikus konstruktor. Inicializálja az adatbázis modellt.
        /// </summary>
        static DataManager()
        {
            _Model = new ELearningEntities();
        }

        /// <summary>
        /// Felhasználó lekérése
        /// </summary>
        /// <param name="userName">Azonosító</param>
        /// <returns>Felhasználó</returns>
        public static EUser GetUser(String userName)
        {
            try
            {
                EUser user = _Model.EUsers.Single(x => x.UserName == userName);
                return user;
            }
            catch (InvalidOperationException)
            {
                throw new UserNameNotExitsException(userName);
            }
        }

        /// <summary>
        /// Felhasználó bejelentkeztetése
        /// </summary>
        /// <param name="userName">Azonosító</param>
        /// <param name="userPass">Jelszó</param>
        /// <returns>Felhasználó</returns>
        public static EUser LogOnUser(String userName, String userPass)
        {
            try
            {
                EUser user = _Model.EUsers.Single(x => x.UserName == userName);
                Byte[] passwordBytes = HashPass(userName, userPass);                
                if (user.UserPass.SequenceEqual(passwordBytes))
                {
                    user.UserLogs.Add(new UserLog
                    {
                        LogType = "UserLogOn",
                        LogTime = DateTime.Now,
                        LogMessage = "User " + user.UserName + " logged in."
                    });
                    _Model.SaveChanges();

                    return user;
                }
                else
                {
                    _Model.AddToUserLogs(new UserLog
                    {
                        UserName = userName,
                        LogType = "UserLogOn",
                        LogTime = DateTime.Now,
                        LogMessage = "User " + user.UserName + " log in failed."
                    });
                    _Model.SaveChanges();
                    throw new UserPassWrongException();
                }
            }
            catch (InvalidOperationException)
            {
                throw new UserNameNotExitsException(userName);
            }
            catch (UserPassWrongException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                throw new DataLoadFailedException("A felhasználó lekérése sikertelen!", ex);
            }
        }

        /// <summary>
        /// Felhasználó regisztrálása
        /// </summary>
        /// <param name="user">Felhasználó</param>
        /// <param name="userPass">Jelszó</param>
        public static EUser RegisterUser(EUser user, String userPass)
        {
            try
            {
                if (_Model.EUsers.Count(x => x.UserName == user.UserName) > 0)
                    throw new UserNameDuplicationException(user.UserName);

                user.UserPass = HashPass(user.UserName, userPass);

                _Model.AddToEUsers(user);
                user.UserLogs.Add(new UserLog
                {
                    LogTime = DateTime.Now,
                    LogType = "UserCreate",
                    LogMessage = "User " + user.UserName + " created."
                });
                _Model.SaveChanges();

                return user;
            }
            catch (UserNameDuplicationException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                throw new DataSaveFailedException("A felhasználó regisztrálása sikertelen!", ex);
            }
        }

        /// <summary>
        /// Jelszó hasítása
        /// </summary>
        /// <param name="userName">Azonosító</param>
        /// <param name="userPass">Jelszó</param>
        /// <returns>Hash</returns>
        private static Byte[] HashPass(String userName, String userPass)
        {
            StringBuilder builder = new StringBuilder(userPass, 64);
            while (builder.Length < 64)
            {
                Int32 n = Convert.ToInt32(Math.Pow(builder.Length * userName.Length, Math.Sqrt(5)));
                if (builder.Length % 2 == 0)
                    builder.Insert(n % builder.Length, userPass[n % userPass.Length]);
                else
                    builder.Insert(n % builder.Length, _PasswordSalt[n % _PasswordSalt.Length]);
            }

            SHA1CryptoServiceProvider coder = new SHA1CryptoServiceProvider();
            return coder.ComputeHash(System.Text.Encoding.UTF8.GetBytes(builder.ToString()));
        }

        /// <summary>
        /// Kurzusok lekérése
        /// </summary>
        /// <returns>Összes kurzus listája</returns>
        public static List<ECourse> GetAllCourses()
        {
            try
            {
                return _Model.ECourses.ToList();
            }
            catch (Exception ex)
            {
                throw new DataLoadFailedException("A kurzusok lekérése sikertelen!", ex);
            }
        }

        /// <summary>
        /// Tananyag létrehozása
        /// </summary>
        /// <param name="model">Tananyag létrehozási nézet modellje</param>
        /// <param name="user">Felhasználó</param>
        /// <returns>Tananyag</returns>
        public static ELecture CreateLecture(LectureViewModel model, EUser user)
        {
            try
            {
                ELecture lecture = new ELecture
                {
                    LectureName = model.Name,
                    LectureDescription = model.Description,
                    LecturePrice = model.Credit,
                    CourseId = model.CourseId
                };
                Byte[] lectureContent = new Byte[model.File.InputStream.Length];
                model.File.InputStream.Read(lectureContent, 0, (int)model.File.InputStream.Length);

                lecture.ELectureContents.Add(new ELectureContent
                {
                    Content = lectureContent
                });

                _Model.AddToELectures(lecture);
                _Model.SaveChanges();

                user.UserLogs.Add(new UserLog
                {
                    LogTime = DateTime.Now,
                    LogType = "LectureCreate",
                    LogMessage = "User " + user.UserName + " created Lecture ID " + lecture.Id + "."
                });
                _Model.SaveChanges();

                return lecture;
            }
            catch (Exception ex)
            {
                throw new DataSaveFailedException("A tananyag létrehozása sikertelen!", ex);
            }
        }

        /// <summary>
        /// Előnézeti kép hozzáadása
        /// </summary>
        /// <param name="lecture">Tananyag</param>
        /// <param name="image">Kép</param>
        /// <param name="user">Felhasználó</param>
        /// <returns>Előnézeti kép</returns>
        public static ELectureImage AddLectureImage(ELecture lecture, Image image, EUser user)
        {
            try
            {
                Byte[] smallImageBytes = ImageToByteArray(CreateSmallImage(image));
                Byte[] largeImageBytes = ImageToByteArray(image);

                ELectureImage lectureImage = new ELectureImage
                {
                    ImageLarge = largeImageBytes,
                    ImageSmall = smallImageBytes
                };
                lecture.ELectureImages.Add(lectureImage);
                _Model.SaveChanges();

                user.UserLogs.Add(new UserLog
                {
                    LogTime = DateTime.Now,
                    LogType = "ImageAdded",
                    LogMessage = "User " + user.UserName + " created Lecture image ID " + lectureImage.ImageId + "."
                });
                _Model.SaveChanges();

                return lectureImage;
            }
            catch (Exception ex)
            {
                throw new DataSaveFailedException("A kép hozzáadása a tananyaghoz sikertelen!", ex);
            }
        }
        /// <summary>
        /// Kép lekérése adott tananyaghoz
        /// </summary>
        /// <param name="lectureId">Tananyag azonosítója</param>
        /// <returns>Kiolvasott előnézeti kép</returns>
        public static ELectureImage GetLectureImage(Int32 id)
        {
            try
            {
                return _Model.ELectureImages.Where(x => x.ImageId == id).Single();
            }
            catch (Exception ex)
            {
                throw new DataLoadFailedException("Előnézeti kép lekérése sikertelen!", ex);
            }
        }

        /// <summary>
        /// Adott tananyag csatolmányának lekérése
        /// </summary>
        /// <param name="lectureId">Tananyag azonosítója</param>
        /// <param name="user">Felhasználó</param>
        /// <returns>Kiolvasott tananyag melléklet</returns>
        public static ELectureContent GetLectureContent(Int32 lectureId, EUser user)
        {
            try
            {
                ELectureContent content = _Model.ELectures.Where(x => x.Id == lectureId).Single().ELectureContents.First();
                user.Credits -= content.ELecture.LecturePrice;
                user.UserLogs.Add(new UserLog
                {
                    LogTime = DateTime.Now,
                    LogType = "LectureDownload",
                    LogMessage = "User " + user.UserName + " downloaded Lecture ID " + content.ELecture.Id + "."
                });
                _Model.SaveChanges();

                return content;
            }
            catch (Exception ex)
            {
                throw new DataLoadFailedException("Előnézeti kép lekérése sikertelen!", ex);
            }
        }

        /// <summary>
        /// Kép átalakítása byte tömbbé
        /// </summary>
        /// <param name="image">Átalakítandó kép</param>
        /// <returns>A képnek megfelelő byte tömb</returns>
        private static Byte[] ImageToByteArray(Image image)
        {
            MemoryStream memoryStream = new MemoryStream();
            image.Save(memoryStream, System.Drawing.Imaging.ImageFormat.Png);
            return memoryStream.ToArray();
        }

        /// <summary>
        /// Kicsinyített kép létrehozása
        /// </summary>
        /// <param name="image">Átalakítandó kép</param>
        /// <returns>A lekicsinyített kép</returns>
        private static Bitmap CreateSmallImage(Image image)
        {
            Int32 width = Convert.ToInt32(100 * image.Width / image.Height);
            Bitmap resultBitmap = new Bitmap(width, 100);
            Graphics graphics = Graphics.FromImage(resultBitmap);
            graphics.DrawImage(image, new Rectangle(0, 0, width, 100));

            graphics.Dispose();
            return resultBitmap;
        }
    }
}